
/* Musical Notation Editor for X, Chris Cannam 1994-96 */
/* Drawing methods for entire stave to MusicTeX stream */

/*#define DEBUG 1*/

#include <stdio.h>
#include "Format.h"
#include "StaveEdit.h"
#include "StavePrivate.h"
#include "Draw.h"
#include <Yawn.h>

#include <SysDeps.h>
#include <Debug.h>

/* THIS IS GOING TO NEED A LOT MORE WORK TO COPE WITH THE NEW TIMESIG SCHEME */

/* This file exports its declarations in Stave.h */


static char *instrumentNumerals[] = {
  "i", "ii", "iii", "iv", "v", "vi", "vii", "viii", "ix",
};

typedef struct _StavePointerRec {
  ItemList current;
  MTime    time;
  Group   *group;
} StavePointerRec, *StavePointerSet;


void StaveMTStartColumn(FILE *file, int i, MTime minLength)
{
  int j;

  Begin("StaveMTStartColumn");

  fprintf(file, "\\%s\n",
	  MTimeToNumber(minLength) > 32 ? "NOTEs" :
	  MTimeToNumber(minLength) > 16 ? "NOTes" :
	  MTimeToNumber(minLength) >  4 ? "NOtes" :
	  MTimeToNumber(minLength) >  2 ? "Notes" : "notes");

  for (j = 0; j < i; ++j) {
    fprintf(file, "&%%\n");
  }

  End;
}


void StaveMTEndColumn(FILE *file)
{
  Begin("StaveMTEndColumn");

  fprintf(file, "\\relax\n\\enotes\n");

  End;
}


MTime StaveWriteColumnToMTFile(FILE *file, MajorStave sp, StavePointerSet set,
			       ClefTag *clefs, TimeSignature **timesigs, int n)
{
  int i;
  MTime length, minLength = longestNoteTime;
  MTime thisStart  = longestMTime;
  MTime startTime  = longestMTime;
  MTime minEndTime = longestMTime;
  Boolean down;
  Boolean started;

  Begin("StaveWriteColumnToMTFile");

  for (i = 0; i < n; ++i) {

    if (set[i].current &&
	set[i].current->item->object_class == GroupClass) {

      set[i].group = (Group *)set[i].current->item;
      set[i].current = set[i].group->group.start;
    }

    if (set[i].current) {
      thisStart = set[i].time;
      if (MTimeLesser(thisStart, startTime)) startTime = thisStart;
    }
  }

  /* return longestMTime as a failed/empty case */

  if (thisStart == longestMTime) Return(longestMTime);

  /* find shortest of candidate notes */

  for (i = 0; i < n; ++i) {
    if (set[i].current && set[i].time == startTime) {
      length = 
	set[i].current->item->methods->get_length(set[i].current->item);
      if (MTimeLesser(length, minLength)) minLength = length;
    }
  }

  started = False;

  /* write only those items that actually start dead on startTime --
     note reversed order of counting, because MusicTeX counts staffs
     from the bottom up and I count them from the top down */

  for (i = n - 1; i >= 0; --i) {

    if (set[i].current && set[i].time == startTime) {

      down = False;

      if (set[i].current->item->object_class == ChordClass) {
	
	/* Tail up if lowest is further below the centre line than
           highest is above it */
	
	if ((4 -
	     set[i].current->item->methods->get_lowest
	     (set[i].current->item)->pitch) <
	    (set[i].current->item->methods->get_highest
	     (set[i].current->item)->pitch - 4)) {
	  down = True;
	}
      }
      
      if (set[i].group != NULL) {

	if (!started) {
	  StaveMTStartColumn(file, n - i - 1, minLength); started = True;
	}

	DrawMTGroupItem
	  (set[i].group, file,
	   (minLength > 32 ? 4 : minLength > 16 ? 2 : minLength > 4 ? 1 : 0),
	   n - i - 1, clefs[i], set[i].current);

      } else {

	/* if clef or timesig, ignore if it's the same one as we had
           anyway (we do this to avoid writing the very first example
           of each twice) */

	if (!(set[i].current->item->object_class == ClefClass &&
	      ((Clef *)set[i].current->item)->clef.clef == clefs[i])) {
	
	  if (!started) {
	    StaveMTStartColumn(file, n - i - 1, minLength); started = True;
	  }

	  set[i].current->item->methods->draw_musicTeX
	    (set[i].current->item, file,
	     (minLength > 32 ? 4 : minLength > 16 ? 2 : minLength > 4 ? 1 : 0),
	     n - i - 1, down, clefs[i], False);
	}
      }      

      if (set[i].current->item->object_class == ClefClass) {
        
	clefs[i] = ((Clef *)set[i].current->item)->clef.clef;
      }

      if (set[i].group && set[i].current == set[i].group->group.end) {

	set[i].group = NULL;
      }
      
      set[i].time = AddMTime
	(set[i].time,
	 set[i].current->item->methods->get_length(set[i].current->item));

      if (MTimeLesser(set[i].time, minEndTime)) minEndTime = set[i].time;      
      set[i].current = iNext(set[i].current);
    }

    if (i > 0 && started) fprintf(file, "\\relax\n&");
  }

  if (started) StaveMTEndColumn(file);

  if (MTimeEqual(minEndTime, zeroTime)) Return(NumberToMTime(1));
  else Return(minEndTime);
}


void StaveMTGetClefsAndTimes(MajorStave sp, int i, ClefTag *clefs,
			     TimeSignature **timesigs)
{
  MajorStaveRec *mstave = (MajorStaveRec *)sp;
  ItemList list;

  Begin("StaveMTGetClefsAndTimes");

  clefs[i] = (ClefTag)10;	/* improbable value hack */
  timesigs[i] = NULL;

  if (BarValid(mstave->bar_list->bars[i])) { /* or just bar existence? */

    timesigs[i] = &mstave->bar_list->bars[i]->bar.time;

    /* naive iterator, don't care about groups */
    for (list = mstave->bar_list->bars[i]->group.start;
	 list && (clefs[i] == (ClefTag)10);
	 list = iNext(list)) {

      if (clefs[i] == (ClefTag)10 && list->item->object_class == ClefClass) {
	clefs[i] = ((Clef *)list->item)->clef.clef;
      }
    }
  }

  if (clefs[i] == (ClefTag)10) clefs[i] = TrebleClef;

  End;
}


extern void DrawMTTimeSignature(TimeSignature *, FILE *, int);

Result StaveWriteMusicTeXToFile(MajorStave sp, Widget w) /*parent for filebox*/
{
  int i;
  int bar;
  FILE *file;
  String message;
  String name;

  ClefTag *clefs;
  TimeSignature **timesigs;
  StavePointerSet set;
  Boolean finished = False;

  MajorStaveRec *mstave = (MajorStaveRec *)sp;
  int staves = mstave->staves;
  MTime duration, barLength;
  StaveEltList barList;

  Begin("StaveWriteMusicTeXToFile");

  if (staves > 9) {

    message = (String)XtMalloc(256);
    sprintf(message,
	    "This piece has %d staffs; I can only handle six.", staves);

    if (YQuery(topLevel, message, 2, 0, 1, "Write first 6 only", "Cancel",
	       "Editor Tools - Export MusicTeX") != 0) {
      XtFree(message);
      Return(Failed);
    } else {
      XtFree(message);
      staves = 6;
    }
  }

  if ((file = YFileGetWriteFile(w, "Editor Tools - Export MusicTeX",
				".tex", "TeX")) == NULL) {
    fclose(file);
    Return(Failed);
  }

  StaveBusyStartCount(mstave->bars + 1);
  StaveBusyMakeCount(0);

  for (i = 0; i < staves; ++i) {
    UnformatItemList((ItemList *)&mstave->music[i], NULL);
    StaveResetFormatting(sp, i);
    StaveFormatBars(sp, i, -1);
  }

  clefs = (ClefTag *)XtMalloc(staves * sizeof(ClefTag));
  timesigs = (TimeSignature **)XtMalloc(staves * sizeof(TimeSignature *));
  set = (StavePointerSet)XtMalloc(staves * sizeof(StavePointerRec));

  barList = mstave->bar_list;
  bar = 1;

  for (i = 0; i < staves; ++i) {
    StaveMTGetClefsAndTimes(sp, i, clefs, timesigs);
  }

  fprintf(file,
	  "%% Generated by Rosegarden\n"
	  "%% MusicTeX generation code: Chris Cannam, 1995\n"
	  "%% Bugs and comments to "
	  "cannam@zands.demon.co.uk\n%%\n"
	  "\\input musicnft\n\\input musictex\n%%\n"
	  "%% Total bars: %d\n%%\n"
	  "%% One instrument per staff\n%%\n"
	  "\\musicsize=16\\relax\n\\rm\n\\def\\nbinstruments{%d}%%\n",
	  mstave->bars, staves);

  for (i = 0; i < staves; ++i) {
    fprintf(file,
	    "%%\n\\def\\instrument%s{%s}%%\n"
	    "\\nbportees%s=1\\relax\n",
	    instrumentNumerals[i],
	    mstave->names[staves-i-1], instrumentNumerals[i]);

    /* bit dubious, putting all this output-implementation here! */
    fprintf(file,
	    "%%\n\\global\\cleftoks%s={%d000}%%\n"
	    "\\metertoks%s={{\\meterfrac{%d}{%d}}{}{}{}}%%\n",
	    instrumentNumerals[i],
	    clefs[staves-i-1] == TrebleClef ? 0 :
	    clefs[staves-i-1] ==  TenorClef ? 4 :
	    clefs[staves-i-1] ==   AltoClef ? 3 : 6,
	    instrumentNumerals[i],
	    timesigs[staves-i-1]->numerator,
	    timesigs[staves-i-1]->denominator);
  }

  fprintf(file, "\\debutmorceau\n%%"
	  "\n\\autolines%% these figures will probably need editing:\n"
	  "{24}%% elementary spacings per bar\n3%% bars per line\n"
	  "8%% staffs per page\n\\relax\n%%\n");

  while (barList && !finished) {

    if (bar > 1) fprintf(file, "\\barre\n");
    fprintf(file, "%%\n%% bar %d\n%%\n", bar);

    barLength = duration = zeroTime;
    finished = True;

    for (i = 0; i < staves; ++i) {

      if (!barList->bars[i]) {	/* this will write too much, in many cases */

	set[i].current = NULL;
	timesigs[i] = NULL;

      } else {

	set[i].group = NULL;
	set[i].time = zeroTime;
	set[i].current = barList->bars[i]->group.start;
	finished = False;

	if (MTimeGreater(barList->bars[i]->methods->get_length
			 (barList->bars[i]), barLength)) {
	  barLength = barList->bars[i]->methods->get_length(barList->bars[i]);
	}

	if (!TimeSignaturesEqual(*timesigs[i], barList->bars[i]->bar.time)) {
	  DrawMTTimeSignature(&barList->bars[i]->bar.time, file, staves-i-1);
	  timesigs[i] = &barList->bars[i]->bar.time;
	}
      }
    }

    do {

      duration = StaveWriteColumnToMTFile
	(file, sp, set, clefs, timesigs, staves);

      for (i = 0; i < staves; ++i) {
	if (set[i].time >= barLength) set[i].current = NULL;
      }
      
      /* longestMTime is returned when all set[i].current are NULL: */

    } while MTimeLesser(duration, longestMTime);

    /*
      (MTimeGreater(duration, zeroTime) && MTimeLesser (duration, barLength));
    */

    StaveBusyMakeCount(bar + 1);
    barList = (StaveEltList)Next(barList);
    bar ++;
  }

  fprintf(file, "%%\n\\hfil\\finmorceau\n\\vfil\\eject\n\\bye\n");

  staveMoved = True;
  StaveRefresh(mstave, -1);
  StaveBusyFinishCount();

  fclose(file);
  name = YFileGetLastFilename(False);
  message = (String)XtMalloc(strlen(name) + 12);
  sprintf(message, "Wrote `%s'.", name);
  (void)YQuery(topLevel, message, 1, 0, 0, "OK", NULL);
  XtFree(message);

  Return(Succeeded);
}


